import requests
import pandas as pd
import io


def elevation(lat=(40, 41), lon=(-65, -60)):
    """
    Function to request data from SRTM30.
    Bathymetry / Topography (SRTM30) is a global bathymetry/topography data product distributed by the USGS EROS data
    center. The data product has a resolution of 30 seconds (roughly 1 km).
    :param lat: tuple with min and max latitude
    :param lon: tuple with min and max longitude
    :return: dataframe
    """

    class ElevationSurface(object):
        def __init__(self):

            # Definine the domain of interest
            minlat = lat[0]
            maxlat = lat[1]
            minlon = lon[0]
            maxlon = lon[1]

            url = 'http://coastwatch.pfeg.noaa.gov/erddap/griddap/usgsCeSrtm30v6.csv?topo[(' + str(maxlat) + '):1:(' + \
                  str(minlat) + ')][(' + str(minlon) + '):1:(' + str(maxlon) + ')]'
            s = requests.get(url).content
            df = pd.read_csv(io.StringIO(s.decode('utf-8')), header=0)
            df.reset_index(drop=True)
            df.drop(0, inplace=True)
            df = df[['latitude', 'longitude', 'topo']].astype(float)
            df.rename(columns={'latitude': 'lat', 'longitude': 'lon', 'topo': 'elev'}, inplace=True)

            self.df = df

        def plot(self):

            import plotly.graph_objects as go
            from numpy import array, unique

            lon_u = unique(self.df.lon)
            lat_u = unique(self.df.lat)

            elev = array([self.df.elev[i: i + len(lon_u)] for i in range(0, len(self.df.elev), len(lon_u))])

            fig = go.Figure(data=[go.Surface(z=elev, x=lon_u, y=lat_u)])
            fig.update_layout(scene=dict(
                xaxis_title='Lon, °',
                yaxis_title='Lat, °',
                zaxis_title='Elevation, m'))

            return fig

    return ElevationSurface()


def point_elev(lat, lon):
    minlat = lat - 0.001
    maxlat = lat + 0.001
    minlon = lon - 0.001
    maxlon = lon + 0.001
    c = elevation(lat=(minlat, maxlat), lon=(minlon, maxlon))
    znew = round(c.df.elev.mean())

    return znew



from io import BytesIO
from urllib.request import urlopen
from zipfile import ZipFile
import os
import lasio


def get_las(number_of_files):
    """
    Function to get well logs data as dataframes from Special Report 006 Athabasca Oil Sands Data
    McMurray/Wabiskaw Oil Sands Deposit (Alberta, Canada). By Alberta Geological Survey (AGS).
    :param number_of_files: int number of files to get (max: 2173)
    :return: a dictionary {'Unique well identifier': dataframe}
    """

    zipfile_url = r'https://ags.aer.ca/document/SPE/SPE_006.zip'        # url to the data as a zip file

    with urlopen(zipfile_url) as resp:
        with ZipFile(BytesIO(resp.read())) as zip_file:
            zip_file.extractall('/tmp/ags_spe')         # extracting files

    datasets_dict = {}      # creating an empty dictionary to store results
    directory = r'/tmp/ags_spe/OilSandsDB/Logs/'
    files_list = os.listdir(directory)

    for filename in files_list[:number_of_files]:

        if filename.endswith(".las") or filename.endswith(".LAS"):

            path = os.path.join(directory, filename)        # get specific path as str
            df = lasio.read(path).df()                      # convert .las to dataframe using lasio

            filename = filename.replace('.LAS', '')
            filename = filename.replace('.las', '')

            # adding the dataframe to the dictionary {'Unique well identifier': dataframe}
            datasets_dict.update({filename: df})

    return datasets_dict



from setuptools import setup
with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='petrodc',
    packages=['petrodc', 'petrodc.npd', 'petrodc.usgs_eros', 'petrodc.ags'],
    version='0.0.6',
    license='LGPL v3',
    description='Petroleum Data Collector',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Juan Camilo Gonzalez Angarita',
    author_email='juan@prowellplan.com',
    keywords='petroleum data collector operations wellbore oil gas',
    classifiers=['Programming Language :: Python :: 3',
                 'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
                 'Natural Language :: English',
                 'Topic :: Scientific/Engineering',
                 'Topic :: Software Development',
                 'Topic :: Software Development :: Libraries',
                 'Topic :: Utilities'],
    install_requires=['pandas', 'requests', 'numpy', 'plotly', 'lasio']
)



import requests
import pandas as pd
import numpy as np


def wellbore(dataset_number):
    """
    Function to request wellbore data from NPD Database.
    :param dataset_number: int to select the dataset
        1: oil samples
        2: NPD ID
        3: lithostratigraphy
        4: history
        5: drilling mud
        6: drill stem tests
        7: documents
        8: cores
        9: core photos
        10: coordinates
        11: casing and leak off
        12: exploration
        13: development
        14: shallow
    :return: a dataframe with the data selected
    """

    data_list = ['with-oil-samples', 'with-npdid', 'with-lithostratigraphy', 'with-history', 'with-drilling-mud',
                 'with-drill-stem-tests', 'with-documents', 'with-cores', 'with-core-photos', 'with-coordinates',
                 'with-casing-and-leak-off', 'exploration', 'development', 'shallow']

    data = data_list[dataset_number - 1]

    return get_dataset(data)


def get_dataset(data):
    """
    looping through all pages, append in JSON dictionary, create dataframe
    This function is based on the Anne Estoppey's code https://github.com/AnneEstoppey/EasyDataTools
    """
    dataset = requests.get('https://hotell.difi.no/api/json/npd/wellbore/' + data + '?page=1').json()

    url_dataset = 'http://hotell.difi.no/api/json/npd/wellbore/' + data

    pages = dataset['pages']
    data_list = []
    for i in range(pages):
        url = url_dataset + f'?page={i + 1}'
        dataset_final = requests.get(url).json()
        entries = len(dataset_final['entries'])

        for entry in range(entries):
            data_list.append(dataset_final['entries'][entry])

    df = pd.DataFrame(data_list)
    df.replace('', np.nan, inplace=True)
    return df
